#######################################################################
#                                                                     #
#                                 Pio                                 #
#                                                                     #
#                    Polyglot Input/Output Library                    #
#                                                                     #
#               EPI Pomdapi, INRIA Paris - Rocquencourt               #
#                                                                     #
#  Copyright 2005-2014 INRIA.                                         #
#  Institut National de Recherche en Informatique et en Automatique.  #
#  All rights reserved. This file is distributed only by permission.  #
#                                                                     #
#  Francois Clement <Francois.Clement@inria.fr>                       #
#  Pierre Weis <Pierre.Weis@inria.fr>                                 #
#                                                                     #
#######################################################################

# $Id: configure_additional_options.profile,v 1.1 2014/03/03 17:12:58 fclement Exp $

AVAILABLE_IMPLEMENTATIONS="c c++ fortran ocaml"

print_additional_options_documentation () { :; }
parse_additional_options () { :; }

process_additional_options () {
    UCSB="${USER_CONFIGURE_SETTINGS_BNAME:?}"
    report_message "Setting supported implementation bindings in $UCSB"
    
    # Compute the list of supported implementations.
    SUPPORTED_IMPLEMENTATIONS=""
    UNSUPPORTED_IMPLEMENTATIONS=""
    for IMPL in $AVAILABLE_IMPLEMENTATIONS; do
        case $IMPL in
        ( c ) COMPILERS="$C_COMPILER";;
        ( c++ ) COMPILERS="$CPP_COMPILER";;
        ( fortran ) COMPILERS="$FORTRAN_COMPILER";;
        ( ocaml ) COMPILERS="$OCAML_BYT_COMPILER $OCAML_BIN_COMPILER";;
        ( * )
            internal_error "unknown implementation \"$IMPL\""
            exit $INTERNAL_ERROR
            ;;
        esac
        for COMP in $COMPILERS; do
            case "$COMP" in
            ( Not_found | False ) SUPPORTED=false;;
            ( * ) SUPPORTED=true; break;;
            esac
        done
        if $SUPPORTED; then
            if test -z "$SUPPORTED_IMPLEMENTATIONS"; then
                SUPPORTED_IMPLEMENTATIONS="$IMPL"
            else
                SUPPORTED_IMPLEMENTATIONS="$SUPPORTED_IMPLEMENTATIONS $IMPL"
            fi
        else
            if test -z "$UNSUPPORTED_IMPLEMENTATIONS"; then
                UNSUPPORTED_IMPLEMENTATIONS="$IMPL"
            else
                UI="$UNSUPPORTED_IMPLEMENTATIONS"
                UNSUPPORTED_IMPLEMENTATIONS="$UI $IMPL"
            fi
        fi
    done
    case "$SUPPORTED_IMPLEMENTATIONS" in
    ( "$AVAILABLE_IMPLEMENTATIONS" ) SUPPORTED_IMPLEMENTATIONS="All";;
    ( "" ) SUPPORTED_IMPLEMENTATIONS="None";;
    esac
    case "$UNSUPPORTED_IMPLEMENTATIONS" in
    ( "" ) UNSUPPORTED_IMPLEMENTATIONS="None";;
    ( "$AVAILABLE_IMPLEMENTATIONS" ) UNSUPPORTED_IMPLEMENTATIONS="All";;
    esac
    
    # Store the current settings for supported implementations.
    add_to_config_file \
        "${USER_CONFIGURE_SETTINGS_NO_EXT:?}" \
        "Available implementations" \
          "AVAILABLE_IMPLEMENTATIONS" "$AVAILABLE_IMPLEMENTATIONS" \
        "Supported implementations" \
          "SUPPORTED_IMPLEMENTATIONS" "$SUPPORTED_IMPLEMENTATIONS" \
        "Unsupported implementations" \
          "UNSUPPORTED_IMPLEMENTATIONS" "$UNSUPPORTED_IMPLEMENTATIONS"
    IER=$?
    if test $IER -ne 0; then exit $IER; fi
}
