(***********************************************************************)
(*                                                                     *)
(*                              Ref-indic                              *)
(*                                                                     *)
(*                        Refinement indicators                        *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright (c) 2013-2014 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: umatrix_nonempty_set.mli,v 1.11 2014/06/10 15:08:33 fclement Exp $ *)

(** {3 Matrix slot nonempty sets} *)

type size = Umatrix_universe.size;;
type element = Umatrix_universe.element;;
          (* = Unatural.t * Unatural.t *)
type uindex = Umatrix_universe.index;;
         (* = element Uindex_universe.element
            = (Unatural.t * Unatural.t) Uindex.t *)
type index = element Uindex_universe.index;;
        (* = (Unatural.t * Unatural.t) Uindex.t Uindex.t *)

type t = private
  {
   universe : Umatrix_universe.t;
   uindices : element Uindex_universe.t;
  }
;;

val same : t -> t -> bool;;
(** Physical equality of nonempty sets. *)

val equal : t -> t -> bool;;
(** Structural equality of nonempty sets. *)

(** {6 Construction functions} *)

val make : size -> t;;

val full : Umatrix_universe.t -> t;;

val of_list1 : Umatrix_universe.t -> uindex Ulist1.t -> t;;

val check_same_full : Upervasives.function_name -> t -> t -> unit;;

(** {6 Access functions} *)

val cardinal : t -> Upervasives.length;;

val choice : t -> index * uindex;;

val uindex : t -> index -> uindex;;
val element : t -> index -> element;;

val array_set : 'a array -> 'a -> index -> uindex -> unit;;

(** {6 Iterators} *)

val iter : (index -> uindex -> unit) -> t -> unit;;
val sub_iter :
  (index -> uindex -> unit) ->
  t -> index -> Upervasives.length -> unit
;;

val fold_left : ('accu -> index -> uindex -> 'accu) -> 'accu -> t -> 'accu;;

val full_iter : (index -> uindex -> element -> unit) -> t -> unit;;
val full_fold_left :
  ('accu -> index -> uindex -> element -> 'accu) -> 'accu -> t -> 'accu
;;
(** When iterating on the set containing all the elements of the universe,
 the access to the value of the elements can be provided by the iterator
 itself at almost zero cost. *)

(** {6 Nonempty subsets} *)

type nonempty_subset =
  {
   support : t;
   indices : index array;
   nonempty_set : t;
  }
;;

val full_index_of_uindex : uindex -> index;;

val subset_of_full : t -> t -> nonempty_subset;;

type dichotomy = nonempty_subset * nonempty_subset;;

val dichotomy : (index -> uindex -> bool) -> t -> dichotomy option;;
